/***************************************************************************
 *   Copyright (C) 2004 by Maciej "Kasprzol" Kasprzyk                      *
 *   katalogue@katalogue.szluug.org                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MYDIRINFO_H
#define MYDIRINFO_H

#include <fstream> // ofstream, cout, cerr
#include <iostream>
#include <vector>
#include <string>

#include <sys/stat.h> // lstat ()
#include <dirent.h> // struct dirent, opendir(), readdir() & so on...

#define USE_EXCEPTIONS 1
/*
using std::vector ;
using std::string ;
using std::cout ;
*/
class MyKatalog ;
class MyDirInfo ;
class MyPlugin ;

/*
hmmm.... spakowany plik bdzie mie swj wasny rozmiar, w rodku bdzie zawiera jaki katalog (o nazwie takiej samej jak plik?)
ktrego rozmiar bdzie rwny rozmiarowi spakowanej zawartoci pliku.
*/

/**
@author Maciej "Kasprzol" Kasprzyk
\brief Base Class for all classes representing indexed elements.

BaseClass is the base class for the classes representing indexed elements.
*/
class BaseClass {
private:
	BaseClass * m_parent ;
public:
	/** The name of the element. */
        std::string name ;
	
	/** An enum describing the type of the element. */
        enum EType { // !!! wprowadzenie zmian w numeracji powoduje e pliki s le wczytywane gdy w plikach zawarte s wartoci liczbowe staych.
		File = 0,		/**< A normal file. */
		LinkToFile = 1,		/**< A link to file. */
		Dir = 2,		/**< Directory. */
		LinkToDir = 3,		/**< A link to directory. */
		CompressedDir = 4,	/**< Compresed directory (e.g. content
		of a zip file). Currently unused. In future will be used by plugins. */
		TopLevelDir = 5,	/**< Top level directory (e.g. the root
		directory of the cd). Contains an additional field - a comment/description
		of the indexed directory structure. */
		DiskGroup = 10,		/**< Disk group. Contains a field with a
		comment/description of the group. */
		Katalog = 11		/**< The most top-level element in a
		catalogue. */
	} ;
	
	/** The type of the element. */
	enum EType type ;
	
	/** Sets the pointer to parent. Used by MyKatalog::operator= */
	void parent (BaseClass * parent) { m_parent = parent ; }
	
	/** Returns the pointer to parent. */
	BaseClass * parent () const { return m_parent ; }
	
	/** Returns the pointer to parent. */
	BaseClass * up () const { return m_parent ; }
	
	/** Returns a string filled with the complete catalogue path of the element.
	* For example: <pre>Catalogue Name/Group Name/Disk Name/Directory1/Directory2/.../DirectoryN/ElementName</pre>
	*/
	/*virtual*/ std::string AbsKataloguePath () const ;
	
	// warning: `class BaseClass' ( i potomne) has virtual functions but non-virtual destructor, ale tym nie naley (?) si przejmowa
	// tylko jedna funkcja wirtualna, ktra nie musi (?) nawet by wirtualna
	/** Constructor of the BaseClass type. Does nothing besides initializing member values.
	* \param n The element name.
	* \param t The element type.
	* \param parent Pointer to parent element.
	*/
	BaseClass (const std::string & name_, EType type_, BaseClass * parent_) : m_parent (parent_), name (name_), type (type_) { }
	
	/** Constructor of type BaseClass. Does nothing besides initializing member values.
	* \param type_ The element type.
	* \param parent_ Pointer to parent element.
	*/
	BaseClass (EType type_, BaseClass * parent_) : m_parent (parent_), type (type_) { }
} ;

namespace MyDirInfoStructures {

/**
@author Maciej "Kasprzol" Kasprzyk
*/
struct MyFileInfoPluginInfo {
	MyPlugin * plugin ;
	const void * PluginData ;
	unsigned int size ; // rozmiar danych plugina
} ;

#ifdef USE_EXCEPTIONS
enum FileReadErrorType {
	NoEndOfFile = 1,
	NoEndOfGroup,
	UnknownChar,
	WrongHeader,
	WrongFileVersion
} ;

struct FileReadException {
	FileReadErrorType type ;
	unsigned char znak ;
	uint pos ;
	FileReadException (FileReadErrorType t, unsigned char c, uint p) : type (t), znak (c), pos (p) {}
};
#endif
} // namespace MyDirInfoStructures {

/**
@author Maciej "Kasprzol" Kasprzyk
\brief A class representing an indexed file.

This class contains all informations about an index file, such as its size, mode, owner id etc.
*/
class MyFileInfo : public BaseClass {
public:
	/** If it is a link, then this is the name of the link target. */
	std::string linkto ;
	
//	string abspath ;

	/** The time of last access. */
	time_t atime ;
	
	/** The time of last modification. */
	time_t mtime ;
	
	/** The time of last status change. */
	time_t ctime ;
	
	/** The id of the file owner. */
	unsigned short uid ;
	
	/** The group id of the file owner. */
	unsigned short gid ;
	/** The file access mode. */
	int mode ;
	/** The file size. */
	unsigned long long size ;
	
	/** */
	std::vector<MyDirInfoStructures::MyFileInfoPluginInfo> plugins ;
//	MyFileInfo () : BaseClass (File, NULL), atime(0), mtime(0), ctime(0), uid(0), gid(0), mode(0), size(0) {}
	/** Constructor of type MyFileInfo. All member varibles are set to 0.
	\param parent_ Parent element (directory) of the file.
	*/
	MyFileInfo (BaseClass * parent_) : BaseClass (File, parent_), atime(0), mtime(0), ctime(0), uid(0), gid(0), mode(0), size(0) {}
protected:
	/** Constructor of type MyFileInfo. Used by MyDirInfo::scan () method.
	\param name The name of the file.
	\param st The information about the file gathered by the <pre>stat</pre> function.
	\param type_ The type of the file (regular file or a symlink to a file).
	\parent parent_ Parent element (directory) of the file.
	*/
	MyFileInfo (const std::string & name, const struct stat * st, EType type_, BaseClass * parent_) ; // constructor only used by MyDirInfo
public:
	MyFileInfo (const std::string & name, const std::string & parentDir, const struct stat * st, EType t, BaseClass * p) ;
	MyFileInfo (const std::string & name, const std::string & parentDir, const struct stat * st, const std::string & link, BaseClass * p) ;
	void PluginsScan (const std::string & filefullpath) ;
} ;

class MyTopLevelDirInfo ;

/**
@author Maciej "Kasprzol" Kasprzyk
*/
class MyDirInfo : public MyFileInfo {
friend class MyTopLevelDirInfo ;
protected:
	MyDirInfo (const std::string & name, const std::string & fullPath, const struct stat * st, MyDirInfo * node_up) : MyFileInfo (name, st, Dir, node_up) { scan (fullPath + "/" + name) ; }
	MyDirInfo (BaseClass * parent) : MyFileInfo (parent) { }
	void scan (const std::string & DirToScan) ;
	void SaveDir (std::ofstream & of) ;
public:
	std::vector <MyFileInfo *> files ;
	std::vector <MyDirInfo *> subdirs ;
//	MyDirInfo (const char * Directory, BaseClass * Up) ;
	MyDirInfo (std::ifstream & ifs, BaseClass * u) throw (MyDirInfoStructures::FileReadException) ;
	~MyDirInfo () ;
} ;

/**
@author Maciej "Kasprzol" Kasprzyk
*/
class MyTopLevelDirInfo : public MyDirInfo {
public:
	std::string comment ;
	MyTopLevelDirInfo (const std::string & Directory, BaseClass * Up) ; // scans the given directory and builds the whole dir tree
	MyTopLevelDirInfo (std::ifstream & ifs, BaseClass * u) throw (MyDirInfoStructures::FileReadException) ;
//	~MyTopLevelDirInfo () ;
	void SaveDir (std::ofstream & of) ;
} ;

class MyKatalog ;

/**
@author Maciej "Kasprzol" Kasprzyk
*/
class MyDiskGroup : public BaseClass {
public:
	std::string comment ;
	std::vector<MyTopLevelDirInfo*> Disks ;
	unsigned long long size () ;
//	MyDiskGroup () : BaseClass (DiskGroup, NULL) { }
	MyDiskGroup (BaseClass * p) : BaseClass (DiskGroup, p) { }
	~MyDiskGroup ()
	{
		std::vector<MyTopLevelDirInfo*>::iterator i ;
		for (i = Disks.begin () ; i != Disks.end () ; i++) {
			delete *i ;
		}
	}
} ;

/**
@author Maciej "Kasprzol" Kasprzyk
*/
class MyKatalog : public BaseClass {
public:
	std::vector<MyTopLevelDirInfo*> Disks ;
	std::vector<MyDiskGroup*> DiskGroups ;
	
	void SaveToFile (const char * filename) ;
	void SavePluginsTable (std::ofstream & of) ;
	void LoadPluginsTable (std::ifstream & ifs) ;
	void LoadFromFile (const char * filename) throw (MyDirInfoStructures::FileReadException) ;
	void destruct () throw ()
	{
		std::vector<MyTopLevelDirInfo*>::iterator i, de = Disks.end () ;
		std::vector<MyDiskGroup*>::iterator j, ge = DiskGroups.end () ;
			for (j = DiskGroups.begin () ; j != ge ; j++) {
				delete *j ;
			}
			for ( i = Disks.begin () ; i != de ; i++) {
				delete *i ;
			}
			DiskGroups.clear () ;
			Disks.clear () ;
	}
	MyKatalog (std::string name = "") throw () : BaseClass (name, Katalog, NULL) { }
	~MyKatalog () { destruct () ; }
	MyKatalog & operator = (const MyKatalog & rhs) ;

private:
	void SaveDiskGroups (std::ofstream & of) ;
	void LoadDiskGroups (std::ifstream & ifs) throw (MyDirInfoStructures::FileReadException) ;
//  void SaveDisks (std::ofstream & of) ;
	void LoadDisks (std::ifstream & ifs) ;
} ;

#endif
